from typing import Optional
from .print_stuff import PrintColor


class Plugin(object):
    """
    Abstract base class for commands that process directives.
    """

    def __init__(self):
        self.l = PrintColor()

    def can_handle(self, directive: str) -> bool:
        """
        Returns true if the Plugin can handle the directive.
        """
        raise NotImplementedError

    def execute(self, directive: str, data: dict) -> dict:
        """
        Executes the directive.
        """
        raise NotImplementedError

    def stats(self, directive: str, data: dict) -> {str: int}:
        """
        Return the stats for the directive
        """
        raise NotImplementedError

    def info(self, directive: str, data: dict) :
        """
        Return info of the directive
        """
        return data.get(directive,dict())




import dns.resolver
from servergrimoire.plugin import Plugin


class DNSLookup(Plugin):
    def can_handle(self, directive: str) -> bool:
        return directive == "dns_lookup"

    def execute(self, directive: str, data: dict) -> dict:
        domain = data["url"]
        array_input = [
            (domain, "A"),
            (domain, "MX"),
            ("mail." + domain, "A"),
            (domain, "NS"),
            (domain, "TXT"),
        ]
        output = {}
        for query, label in array_input:
            output[label]=[]
            try:
                for rdata in dns.resolver.resolve(query, label):
                    output[label].append(str(rdata))
            except:
                self.l.info(f"Not found {label} for {domain}")
        return output

    def stats(self, directive: str, data: dict) -> {str: int}:
        return dict()



from tabulate import tabulate

class ColorClass:
    def __init__(self, palet=None):
        if palet is None:
            palet = dict()
        self.info_color = palet.get("okblue", "\033[94m")
        self.debug_color = palet.get("okgreen", "\033[92m")
        self.warning_color = palet.get("warning", "\033[93m")
        self.fail_color = palet.get("fail", "\033[91m")
        self.end_color = palet.get("endc", "\033[0m")

    def info(self, message: str) -> str:
        raise NotImplementedError

    def debug(self, message: str) -> str:
        raise NotImplementedError

    def warning(self, message: str) -> str:
        raise NotImplementedError

    def fail(self, message: str) -> str:
        raise NotImplementedError

class StrColor(ColorClass):
    def __printer__(self, message: str, color: str = None) -> str:
        return f"{color}{message}{self.end_color}"

    def info(self, message: str) -> str:
        return self.__printer__(message, self.info_color)

    def debug(self, message: str) -> str:
        return self.__printer__(message, self.debug_color)

    def warning(self, message: str) -> str:
        return self.__printer__(message, self.warning_color)

    def fail(self, message: str) -> str:
        return self.__printer__(message, self.fail_color)


class PrintColor(ColorClass):
    def __init__(self, palet=None):
        super().__init__(palet)
        self.s = StrColor(palet)

    def info(self, message: str) -> str:
        return self.s.info(message)

    def debug(self, message: str) -> str:
        return self.s.debug(message)

    def warning(self, message: str) -> str:
        return self.s.warning(message)

    def fail(self, message: str) -> str:
        return self.s.fail(message)



import datetime
import whois
from servergrimoire.plugin import Plugin


class DNSChecker(Plugin):
    def can_handle(self, directive: str) -> bool:
        return directive == "dns_checker"

    @staticmethod
    def get_directives() -> [str]:
        return ["dns_checker"]

    def execute(self, directive: str, data: dict) -> dict:
        w = whois.whois(data["url"])
        if w["domain_name"] is None:
            output_strng = {
                "status": "XX",
                "creation": "Error trying to connect to socket",
                "expiration": "Error trying to connect to socket",
            }
        else:
            flag = w.expiration_date >= (
                datetime.datetime.now() - datetime.timedelta(days=30)
            )

            if flag:
                output_strng = {
                    "status": "OK",
                    "creation": f"{w.creation_date}",
                    "expiration": f"{w.expiration_date}",
                }
            else:
                output_strng = {
                    "status": "XX",
                    "creation": f"{w.creation_date}",
                    "expiration": f"{w.expiration_date}",
                }

        self.l.info(output_strng)
        return output_strng

    def stats(self, directive: str, data: dict) -> {str: int}:
        stat = {"OK": 0, "KO": 0, "XX": 0}
        stat[data[directive]["status"]] = 1
        return stat



import json
from pprint import pprint

from tabulate import tabulate

from servergrimoire.configmanager import ConfigManager
from servergrimoire.operation.dnschecker import DNSChecker
from servergrimoire.operation.dnslookup import DNSLookup
from servergrimoire.operation.sslverify import SSLVerify
from servergrimoire.plugin import Plugin


class GrimoirePage:
    def __init__(self, path):
        self.path = path
        self.setting_manager = ConfigManager(path)

        try:
            with open(self.setting_manager.data_path) as f:
                self.data = json.load(f)
        except FileNotFoundError:
            with open(self.setting_manager.data_path, "w") as f:
                json.dump({}, f)
            self.data = {}

    def __get_directives_and_class(self) -> dict:
        """
        Return a dict with all directories and theire class
        """
        dict_directives = {}
        for _class in self.__get_directives_class():
            for e in _class.get_directives():
                dict_directives[e] = _class
        return dict_directives

    def __get_urls(self):
        """
        Return a array with all urls and theire class
        """
        return self.__get_directives_class().keys()

    def __get_directives_class(self) -> [Plugin]:
        return [DNSChecker, DNSLookup, SSLVerify]

    def __get_directives_str(self) -> [str]:
        """
        Return all directive into a array
        """
        arr_directives = []
        for _class in self.__get_directives_class():
            for e in _class.get_directives():
                arr_directives.append(e)
        return arr_directives

    def __get_urls_all(self) -> [str]:
        """
        Return all urls into a array
        """
        try:
            return self.data["server"].keys()
        except:
            return []

    def run(self, command=None, url=None):
        """
        Launch command for plugin
        """
        map_command = self.__get_directives_and_class()
        if command is None:
            command_to_run = self.__get_directives_str()
        else:
            command_to_run = [command]
        url_to_run = None
        if url is None:
            url_to_run = self.__get_urls_all()
        else:
            url_to_run = [url]

        for url in url_to_run:
            for command in command_to_run:
                cl = map_command[command]()
                self.data["server"][url][command] = cl.execute(
                    directive=command, data=self.data["server"][url]
                )

        with open(self.setting_manager.data_path, "w") as json_file:
            json.dump(self.data, json_file)

    def stats(self, command=None, url=None) -> None:
        """
        Launch stats command for plugin
        """
        map_command = self.__get_directives_and_class()
        if command is None:
            command_to_run = self.__get_directives_str()
        else:
            command_to_run = [command]
        if url is None:
            url_to_run = self.__get_urls_all()
        else:
            url_to_run = [url]

        printable = {}
        for command in command_to_run:
            printable[command] = {}
            for url in url_to_run:
                all = map_command[command]().stats(command, self.data["server"][url])
                for key in all.keys():
                    printable[command][key] = printable[command].get(key, 0) + int(
                        all[key]
                    )

        for command in printable.keys():
            message = [(k, v) for k, v in printable[command].items()]
            head = [command, ""]
            print(tabulate(message, head, tablefmt="pipe"))
            print()

    def info(self, command=None, url=None) -> None:
        """
        Launch info command for plugin
        """
        map_command = self.__get_directives_and_class()
        if command is None:
            command_to_run = self.__get_directives_str()
        else:
            command_to_run = [command]
        if url is None:
            url_to_run = self.__get_urls_all()
        else:
            url_to_run = [url]

        printable = {}
        for command in command_to_run:
            printable[command] = {}
            for url in url_to_run:
                all = map_command[command]().info(directive=command, data=self.data["server"][url])
                printable[command][url]=all

        pprint(printable)


    def add(self, url) -> bool:
        """
        Add command for url
        """
        for e in url:
            if self.data.get("server") is None:
                self.data["server"] = {}
            if self.data["server"].get(e) is None:
                self.data["server"][e] = {"url": e}
        with open(self.setting_manager.data_path, "w") as json_file:
            json.dump(self.data, json_file)

    def remove(self, url=None) -> bool:
        """
        Remove command for url
        """
        self.data["server"].pop(url, None)
        with open(self.setting_manager.data_path, "w") as json_file:
            json.dump(self.data, json_file)



import datetime
import socket
import ssl
from servergrimoire.plugin import Plugin

BROKEN_RESPONSE = {"status": "KO", "expired": "****-**-** **:**:**"}


class SSLVerify(Plugin):
    def can_handle(self, directive: str) -> bool:
        return directive == "ssl_check"

    def __ssl_valid_time_remaining(self, hostname: str) -> datetime.datetime:
        ssl_date_fmt = r"%b %d %H:%M:%S %Y %Z"

        context = ssl.create_default_context()
        conn = context.wrap_socket(
            socket.socket(socket.AF_INET), server_hostname=hostname,
        )

        self.l.debug("Connect to {}".format(hostname))
        conn.connect((hostname, 443))
        ssl_info = conn.getpeercert()
        # parse the string from the certificate into a Python datetime object
        return datetime.datetime.strptime(ssl_info["notAfter"], ssl_date_fmt)

    def execute(self, directive: str, data: dict) -> dict:
        """Return test message for hostname cert expiration."""
        limit = datetime.datetime.now() + datetime.timedelta(days=30)
        output_strng = None
        try:
            will_expire_in = self.__ssl_valid_time_remaining(data["url"])
        except FileNotFoundError as e:
            output_strng = BROKEN_RESPONSE
        except socket.gaierror as e:
            output_strng = BROKEN_RESPONSE
        except ssl.CertificateError as e:
            output_strng = BROKEN_RESPONSE
        except ssl.SSLError as e:
            output_strng = BROKEN_RESPONSE
        except socket.timeout as e:
            output_strng = BROKEN_RESPONSE
        else:
            if will_expire_in is None:
                output_strng = BROKEN_RESPONSE
            elif will_expire_in < limit:
                output_strng = {"status": "KO", "expired": str(will_expire_in)}
            elif will_expire_in < limit:
                output_strng = {"status": "XX", "expired": str(will_expire_in)}
            else:
                output_strng = {"status": "OK", "expired": str(will_expire_in)}
        self.l.info(f"{directive} return {output_strng}")
        return output_strng

    def stats(self, directive: str, data: dict) -> {str: int}:
        stat = {"OK": 0, "KO": 0, "XX": 0}
        stat[data[directive]["status"]] = 1
        return stat



import json
import os
from pathlib import Path


class ConfigManager:
    def __init__(self, path):
        self.config = {}
        if path is None:
            path = Path.home() / ".servergrimoire_config"
        self.path = path

        if not os.path.exists(path):
            self.create_default()
        else:
            with open(path) as data_file:
                self.config = json.load(data_file)

    def create_default(self) -> dict:
        with open(Path(self.path), "w") as outfile:
            self.config["data_path"] = str(Path.home() / ".servergrimoire_data")
            json.dump(self.config, outfile)
        return self.config

    @property
    def data_path(self):
        return self.config["data_path"]

    @data_path.setter
    def data_path(self, var):
        self.config["data_path"] = var

    @data_path.getter
    def data_path(self):
        if self.config.get("data_path", None) is None:
            self.config["data_path"] = Path.home() / ".servergrimoire_data"
        return self.config["data_path"]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['servergrimoire', 'servergrimoire.operation', 'servergrimoire.script']

package_data = \
{'': ['*']}

install_requires = \
['Click>=7.1.2,<8.0.0',
 'dnspython>=2.0.0,<3.0.0',
 'pyOpenssl>=19.1.0,<20.0.0',
 'python-whois>=0.7.3,<0.8.0',
 'requests>=2.24.0,<3.0.0',
 'tabulate>=0.8.7,<0.9.0']

setup_kwargs = {
    'name': 'servergrimoire',
    'version': '1.0.1',
    'description': 'Package for record and store info about servers and their stuffs',
    'long_description': None,
    'author': 'Fundor333',
    'author_email': 'fundor333@fundor333.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
